function [CSVData] = import_csv_file(filename, startRow, endRow)
%IMPORT_CSV_FILE Import numeric data from a groundtruth file to the
% struct CSVDATA.
%   [CSVDATA] = IMPORT_CSV_FILE(FILENAME, STARTROW, ENDROW)
%  Reads data from the groundtruth file FILENAME
%
%   Parameters:
%     FILENAME  the filename
%     START_ROW the first data row to be read. The header is always read
%               from the first row.
%     END_ROW   the last data row to be read.
%
%   Outputs:
%     CSVDATA struct containing the data. The fieldnames are written from
%     the filename header (first row)
%
% Example:
%   [CSVDATA] = import_csv_file('groundtruth.csv',1, 197);
%
% Copyright (c) MVTec Software GmbH.  All rights reserved.
%
% All use of this software is subject to the terms of the revised BSD
% license.  You should have received a copy of this license along
% with this source code in a file named "license.txt."

%% First we read the header
fileID = fopen(filename,'r');
CellHeader = textscan(fileID,'%s',1);
header = strsplit(cell2mat(CellHeader{1,1}),';');
fclose(fileID);

%% The data is everything from the second row onwards
delimiter = ';';
if nargin<=2
    startRow = 2;
    endRow = inf;
end

%% Format for each line of text:
% For more information, see the TEXTSCAN documentation.
formatSpec = strcat(repmat('%s',1,length(header)),'%[^\n\r]');

%% Open the text file.
fileID = fopen(filename,'r');

%% Read columns of data according to the format.
% This call is based on the structure of the file used to generate this
% code. If an error occurs for a different file, try regenerating the code
% from the Import Tool.
dataArray = textscan(fileID, formatSpec, endRow(1)-startRow(1)+1, 'Delimiter', delimiter, 'HeaderLines', startRow(1)-1, 'ReturnOnError', false, 'EndOfLine', '\r\n');
for block=2:length(startRow)
    frewind(fileID);
    dataArrayBlock = textscan(fileID, formatSpec, endRow(block)-startRow(block)+1, 'Delimiter', delimiter, 'HeaderLines', startRow(block)-1, 'ReturnOnError', false, 'EndOfLine', '\r\n');
    for col=1:length(dataArray)
        dataArray{col} = [dataArray{col};dataArrayBlock{col}];
    end
end

%% Close the text file.
fclose(fileID);

%% Allocate imported array to column variable names
for idx = 1:length(header)
    % We need to get rid of the '-'
    CSVData.(regexprep(header{idx},'[-/\s]','')) = dataArray{:, idx};
end

