function [box_param] = convert_region2box_param(region)
%CONVERT_REGION2BOX_PARAM
%   Convert vot region points to bounding box coordinates
%
%   Parameters:
%     REGION box coordinates [row1,col1,row2,col2,row3,col3,row4,col4]
%            or four corner point parameterization (these are converted to
%            the vot2016 box coordinates automatically)
%
%   Outputs:
%     BOX_PARAM box coordinates [row,col,width,height,phi]
%
% Copyright (c) MVTec Software GmbH.  All rights reserved.
%
% All use of this software is subject to the terms of the revised BSD
% license.  You should have received a copy of this license along
% with this source code in a file named "license.txt."

%% Calculate region
% Get the corner points - In case its an axis aligned rectangle,
% convert it to the vot2016 region format first...
if (size(region,2) == 4)
    region = [region(:,1),region(:,2),region(:,1)+region(:,3),region(:,2),...
              region(:,1)+region(:,3),region(:,2)+region(:,4),...
              region(:,1),region(:,2)+region(:,4),...
              region(:,1),region(:,2)];
end

col1 = region(:,1);
row1 = region(:,2);
col2 = region(:,3);
row2 = region(:,4);
col3 = region(:,5);
row3 = region(:,6);
col4 = region(:,7);
row4 = region(:,8);

% The center (row,col) can be calculated directly from the four points
row = (row1+row2+row3+row4)/4.0;
col = (col1+col2+col3+col4)/4.0;

% So can the width and the height:
length1 = sqrt((row1 - row2).*(row1 - row2) + (col1-col2).*(col1-col2)) + 1;
length2 = sqrt((row2 - row3).*(row2 - row3) + (col2-col3).*(col2-col3)) + 1;


IDX = (length1 < length2);
% Swap length1 and length2 where length1 < length2
tmp = length1;
length1(IDX) = length2(IDX);
length2(IDX) = tmp(IDX);

hor(IDX)  = (col3(IDX) - col2(IDX));
vert(IDX) = (row2(IDX) - row3(IDX));

hor(~IDX)  = (col2(~IDX) - col1(~IDX));
vert(~IDX) = (row1(~IDX) - row2(~IDX));

% Calculate phi and length and height accordingly
phi = atan2(vert,hor);
phi(phi > pi/2) = phi(phi > pi/2) - pi;
phi(phi <= -pi/2) = phi(phi <= -pi/2) + pi;

%% Set the output
% HDevelop and Matlab have swapped image axes and HDevelop axes start at (0,0)
% => (row_hdev + 1, col_hdev + 1) = (col_matlab, row_matlab)
box_param = [row - 1, col - 1, length1, length2, rad2deg(phi')];

% Consolidate the angles
box_param = consolidate_box_param(box_param);
