#!/usr/bin/env python

# --------------------------------------------------------
# File: write_python_tracker_to_csv.py
# Copyright (c) MVTec Software GmbH.  All rights reserved.
#
# All use of this software is subject to the terms of the revised BSD
# license.  You should have received a copy of this license along
# with this source code in a file named "license.txt."
# --------------------------------------------------------

"""
This script loads the trackers from the passed tracker output csv
file (tracker_boxes.csv per default) and generates the box_overlap.csv
file (box_overlaps_new.csv per default). Only the overlaps of trackers which
appear in input file but not in the box_overlap file are computed and
added to csv

See readme.txt for installation instructions before running.
"""

import argparse
import mvtec_csv
import mvtec_util
from mvtec_config import cfg, cfg_from_file
import os
import glob
import time

fmt_str = "{0:23} {1}"

if __name__ == '__main__':

    print 'Write box overlaps to csv'

    parser = argparse.ArgumentParser(
        description='Write tracker IoUs to csv',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument("--cfg", default=None,
                        help="""configuration file to overwrite default config
                                under code/python/lib/mvtec_config.py""")
    args = parser.parse_args()

    if args.cfg is not None:
        cfg_from_file(args.cfg)
    image_folder = os.path.join(cfg.BASE_DIR, cfg.IMAGE_DIR)
    annot_folder = os.path.join(cfg.BASE_DIR, cfg.ANNOT_DIR)

    ###########################################################################
    # We read the theoretical tracker values from cfg.TRACKER_CSV and
    # cfg.OVERLAP_CSV. If you want to write to these files, set the variables
    # output_tracker_csv and output_overlaps_csv accordingly!
    output_overlaps_csv = 'box_overlaps_new.csv'
    theoretical_tracker_name = 'theoretical_box_axis_aligned'
    ###########################################################################

    # Get sorted list of sequences
    sequence_folders = [d for d in os.listdir(image_folder) if
                        os.path.isdir(os.path.join(image_folder, d))]
    sequence_folders.sort()
    num_sequences = len(sequence_folders)
    all_compute_times = []
    # We loop over all sequences:
    for seq_idx in range(num_sequences):
        seq_name = sequence_folders[seq_idx]
        print fmt_str.format('Sequence:', seq_name)
        annot_dir = os.path.join(annot_folder, seq_name)

        # Get sorted list of available annotations
        os.chdir(annot_dir)
        annot_files = glob.glob('*.png')
        annot_files = [annot_file.split('.')[0] for annot_file in annot_files]
        annot_files.sort(key=int)
        annot_files = [annot_file + '.png' for annot_file in annot_files]
        if len(annot_files) == 0:
            raise Exception("Could not find annotations under {}".
                            format(annot_dir))

        # Get overlaps and tracker-box csv-files (located in the image-folder)
        os.chdir(os.path.join(image_folder, seq_name))
        overlaps_csv = cfg.OVERLAP_CSV
        tracker_csv = cfg.TRACKER_CSV
        tracker_df = mvtec_csv.read_csv(tracker_csv)
        tracker_overlap_df = mvtec_csv.read_csv(overlaps_csv)

        # Check which tracker are not yet in output_overlaps_csv
        new_trackers = []
        for tracker_name in list(tracker_df):
            # We are only interested in trackers...
            if not tracker_name.find('tracker_') == -1:
                # ignore the *_iou_abs and *_iou_rel stuff
                if not any(tracker_name in s for s in
                           list(tracker_overlap_df)):
                    new_trackers.append(tracker_name)

        if len(new_trackers) == 0:
            print "No new tracker identified"
            continue

        print fmt_str.format("Calculating overlaps for: ", new_trackers)

        for tracker_name in new_trackers:
            new_tracker_name = tracker_name
            ###################################################################
            polygons = []
            # We go through the sequence and collect the boxes (in polygon
            # representation)
            for img_idx in range(len(annot_files)):
                # We get box from theoretical box_rot tracker
                box_params = mvtec_csv.get_tracker_box(
                    tracker_df, tracker_name, img_idx)

                # Convert into 4-point polygon representation
                polygon = mvtec_util.convert_box2polygon(box_params)
                polygons.append(polygon)

            ###################################################################
            # Now that we have the tracker output, we can calculate IoUs and
            # finally write to csv-files.
            # Note: This process is a lot faster using Halcon:
            # Halcon avg 0.025 ms vs Python 3.6 ms
            boxes = []
            ious_abs = []
            ious_rel = []
            compute_times = []
            # We go through the sequence, calculate IoUs and convert tracker
            # boxes to box representation
            for img_idx in range(len(annot_files)):
                # Get segmentation as mask
                segmentation = mvtec_util.\
                               read_segmentation(annot_dir,
                                                 annot_files[img_idx])
                image_shape = (segmentation.shape[1], segmentation.shape[0])
                box_mask = mvtec_util.polygon2mask(polygons[img_idx],
                                                   image_shape)
                # Calculate absolute IoU
                tic = time.clock()
                abs_iou = mvtec_util.get_iou(box_mask, segmentation)
                toc = time.clock()
                compute_times.append(toc - tic)
                all_compute_times.append(toc - tic)
                ious_abs.append(abs_iou)

                # Get box-mask of corresponding theoretical tracker
                theoretical_box_params = mvtec_csv.get_tracker_box(
                    tracker_df,
                    theoretical_tracker_name,
                    img_idx
                                         )
                # Convert into 4-point polygon representation
                polygon_theoretical = mvtec_util.\
                    convert_box2polygon(theoretical_box_params)
                theoretical_mask = mvtec_util.polygon2mask(polygon_theoretical,
                                                           image_shape)
                # Calculate relative IoU - should be 1.0 in this example
                ious_rel.append(mvtec_util.get_iou(box_mask, theoretical_mask))
                # Finally convert polygon to box_params (we always write to
                # csv in box_param representation)
                boxes.append(mvtec_util.convert_polygon2box(
                    polygons[img_idx]))

            # Now that we have collected everything, we append columns to the
            # csv-files
            mvtec_csv.add_tracker_ious_to_csv(overlaps_csv,
                                              new_tracker_name,
                                              ious_abs,
                                              ious_rel,
                                              output_overlaps_csv)
            # If we have multiple tracker to write, make sure they all land
            # in the new output file...
            if (len(new_trackers) > 1):
                overlaps_csv = output_overlaps_csv

            # Print the average IoU computation time over this sequence
            print fmt_str.format("Avg IoU compute_time:",
                                 sum(compute_times) / len(compute_times))

        #######################################################################
        # Print the average IoU computation time over all sequences
        print fmt_str.format("Total Avg IoU compute_time:",
                             sum(all_compute_times) / len(all_compute_times))
