#!/usr/bin/env python

# --------------------------------------------------------
# File: write_python_tracker_to_csv.py
# Copyright (c) MVTec Software GmbH.  All rights reserved.
#
# All use of this software is subject to the terms of the revised BSD
# license.  You should have received a copy of this license along
# with this source code in a file named "license.txt."
# --------------------------------------------------------

"""
This script shows how you can calculate absolute and relative IoUs of your
tracker using python. The results will be appended as columns to csv-files.
Additionally the script writes out the tracker boxes per frame to csv in our
5-param representation.

For convenience, the example tracker just outputs the theoretical tracker
boxes of box_rot. We name it 'awesome'.

We indicate places where you could insert your own tracker inside our code.

See readme.txt for installation instructions before running.
"""

import argparse
import mvtec_csv
import mvtec_util
from mvtec_config import cfg, cfg_from_file
import os
import glob
import time

fmt_str = "{0:23} {1}"

if __name__ == '__main__':

    print 'Write new tracker IoUs and boxes to csv'

    parser = argparse.ArgumentParser(
        description='Write new tracker IoUs and boxes to csv',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument("--cfg", default=None,
                        help="""configuration file to overwrite default config
                                under code/python/lib/mvtec_config.py""")
    args = parser.parse_args()

    if args.cfg is not None:
        cfg_from_file(args.cfg)
    image_folder = os.path.join(cfg.BASE_DIR, cfg.IMAGE_DIR)
    annot_folder = os.path.join(cfg.BASE_DIR, cfg.ANNOT_DIR)

    ###########################################################################
    # PLEASE CHANGE THE FOLLOWING VARIABLES ACCORDING TO YOUR NEEDS
    # This needs to be changed according to the abilities of your tracker
    theoretical_tracker_name = 'theoretical_box_rot'

    # We read the theoretical tracker values from cfg.TRACKER_CSV and
    # cfg.OVERLAP_CSV. If you want to write to these files, set the variables
    # output_tracker_csv and output_overlaps_csv accordingly!
    output_overlaps_csv = 'box_overlaps_new.csv'
    output_tracker_csv = 'tracker_boxes_new.csv'

    # Give a unique name to the new tracker
    new_tracker_name = 'awesome'
    ###########################################################################

    # Get sorted list of sequences
    sequence_folders = [d for d in os.listdir(image_folder) if
                        os.path.isdir(os.path.join(image_folder, d))]
    sequence_folders.sort()
    num_sequences = len(sequence_folders)
    all_compute_times = []
    # We loop over all sequences:
    for seq_idx in range(num_sequences):
        seq_name = sequence_folders[seq_idx]
        print fmt_str.format('Sequence:', seq_name)
        annot_dir = os.path.join(annot_folder, seq_name)
        # Get sorted list of available annotations
        os.chdir(annot_dir)
        annot_files = glob.glob('*.png')
        annot_files = [annot_file.split('.')[0] for annot_file in annot_files]
        annot_files.sort(key=int)
        annot_files = [annot_file + '.png' for annot_file in annot_files]

        # Get overlaps and tracker-box csv-files (located in the image-folder)
        os.chdir(os.path.join(image_folder, seq_name))
        overlaps_csv = cfg.OVERLAP_CSV
        tracker_csv = cfg.TRACKER_CSV
        tracker_df = mvtec_csv.read_csv(tracker_csv)

        #######################################################################
        # HERE GOES THE CODE THAT INITIALIZES AND RUNS THE TRACKER ON THE
        # SEQUENCE.
        # PLEASE REPLACE THIS PART BY YOUR TRACKER CODE OR READ THE
        # BOX-PARAMETERS FROM A FILE HERE!
        # OUR TRACKER JUST GETS THE BOXES FROM THE CSV-FILE INSTEAD.
        #######################################################################
        awesome_polygons = []
        # We go through the sequence and collect the boxes (in polygon
        # representation)
        for img_idx in range(len(annot_files)):
            # We get box from theoretical box_rot tracker
            awesome_box_params = mvtec_csv.get_tracker_box(
                tracker_df,
                theoretical_tracker_name,
                img_idx
                                 )
            # Convert into 4-point polygon representation
            polygon_awesome = mvtec_util.\
                convert_box2polygon(awesome_box_params)
            awesome_polygons.append(polygon_awesome)

        #######################################################################
        # Now that we have the tracker output, we can calculate IoUs and
        # finally write to csv-files.
        # Note: This process is a lot faster using Halcon:
        # Halcon avg 0.025 ms vs Python 3.6 ms
        awesome_boxes = []
        awesome_ious_abs = []
        awesome_ious_rel = []
        compute_times = []
        # We go through the sequence, calculate IoUs and convert tracker boxes
        # to box representation
        for img_idx in range(len(annot_files)):
            # Get segmentation as mask
            segmentation = mvtec_util.read_segmentation(annot_dir,
                                                        annot_files[img_idx])
            image_shape = (segmentation.shape[1], segmentation.shape[0])
            box_mask = mvtec_util.polygon2mask(awesome_polygons[img_idx],
                                               image_shape)
            # Calculate absolute IoU
            tic = time.clock()
            abs_iou = mvtec_util.get_iou(box_mask, segmentation)
            toc = time.clock()
            compute_times.append(toc - tic)
            all_compute_times.append(toc - tic)
            awesome_ious_abs.append(abs_iou)

            # Get box-mask of corresponding theoretical tracker
            theoretical_box_params = mvtec_csv.get_tracker_box(
                tracker_df,
                theoretical_tracker_name,
                img_idx
                                     )
            # Convert into 4-point polygon representation
            polygon_theoretical = mvtec_util.\
                convert_box2polygon(theoretical_box_params)
            theoretical_mask = mvtec_util.polygon2mask(polygon_theoretical,
                                                       image_shape)
            # Calculate relative IoU - should be 1.0 in this example
            awesome_ious_rel.append(mvtec_util.get_iou(box_mask,
                                                       theoretical_mask))
            # Finally convert polygon to box_params (we always write to csv in
            # box_param representation)
            awesome_boxes.append(mvtec_util.convert_polygon2box(
                awesome_polygons[img_idx]))

        # Now that we have collected everything, we append columns to the
        # csv-files
        mvtec_csv.add_tracker_ious_to_csv(overlaps_csv,
                                          new_tracker_name,
                                          awesome_ious_abs,
                                          awesome_ious_rel,
                                          output_overlaps_csv)
        mvtec_csv.add_tracker_boxes_to_csv(tracker_csv,
                                           new_tracker_name,
                                           awesome_boxes,
                                           output_tracker_csv)
        # Print the average IoU computation time over this sequence
        print fmt_str.format("Avg IoU compute_time:",
                             sum(compute_times) / len(compute_times))

        #######################################################################
    # Print the average IoU computation time over all sequences
    print fmt_str.format("Total Avg IoU compute_time:",
                         sum(all_compute_times) / len(all_compute_times))
