#!/usr/bin/env python

"""MVTec Python Library for Evaluation of Tracker Accuracy

This file contains the configuration of all python scripts. In particular, the
location of data and annotations, as well as several settings of the app have
to be provided here.

Copyright (c) MVTec Software GmbH.  All rights reserved.

All use of this software is subject to the terms of the revised BSD
license.  You should have received a copy of this license along
with this source code in a file named "license.txt."
"""

import yaml
import numpy as np
from easydict import EasyDict as edict
import os

__C = edict()

# Get config using:
#   from mvtec_config import cfg

cfg = __C


# GENERAL SETTINGS

# Location of this file
__C.LIB_DIR = os.path.abspath(__file__)

# Base folder
__C.BASE_DIR = os.path.abspath(os.path.join(__C.LIB_DIR, os.pardir, os.pardir,
                                            os.pardir, os.pardir))

# Directory, where the sequence-folders with images are located
# (relative to __C.BASE_DIR!)
__C.IMAGE_DIR = 'data/DAVIS/DAVIS/JPEGImages/480p'

# Directory, where the sequence-folders with annotations are located
# (relative to __C.BASE_DIR!)
__C.ANNOT_DIR = 'data/DAVIS/DAVIS/Annotations/480p'

# Name of the csv-file holding the tracker-boxes (one inside each sequence
# image folder)
__C.TRACKER_CSV = 'tracker_boxes.csv'

# Name of the csv-file holding the tracker-ious (one inside each sequence
# image folder)
__C.OVERLAP_CSV = 'box_overlaps.csv'

# APP SETTINGS

# Available theoretical tracker
# Inside the csv-files they have to be named 'theoretical_' + tracker_name,
# e.g. 'theoretical_box_rot'
__C.THEORETICAL_TRACKERS = ['box_no_scale', 'box_axis_aligned', 'box_rot']

# Available real tracker
# Inside the csv-files they have to be named 'tracker_' + tracker_name,
# e.g. 'tracker_awesome'
__C.REAL_TRACKERS = ['kcf', 'dsst', 'ccot', 'ant', 'l1apg', 'lgt']

# (Maximum) size [width, height] of stream-images in app.
# SIZES larger than [854, 480] are not supported. Note that images smaller than
# this size are left unaffected!
__C.STREAM_SIZE = [854, 480]

# Size of the IoU plots [width, height] (Default: [4.5, 4])
__C.PLOT_SIZE = [4.5, 4]

def _merge_a_into_b(a, b):
    """Merge config dictionary a into config dictionary b, clobbering the
    options in b whenever they are also specified in a.
    """
    if not isinstance(a, edict):
        return

    for k, v in a.iteritems():
        # a must specify keys that are in b
        if k not in b.keys():
            raise KeyError('{} is not a valid config key'.format(k))

        # the types must match, too
        old_type = type(b[k])
        if not isinstance(v, old_type):
            if isinstance(b[k], np.ndarray):
                v = np.array(v, dtype=b[k].dtype)
            else:
                raise ValueError(('Type mismatch ({} vs. {}) '
                                  'for config key: {}').format(type(b[k]),
                                                               type(v), k))

        # recursively merge dicts
        if isinstance(v, edict):
            try:
                _merge_a_into_b(a[k], b[k])
            except:
                print('Error under config key: {}'.format(k))
                raise
        else:
            b[k] = v
            setattr(b, k, v)


def cfg_from_file(filename):
    """Load a config file and merge it into the default options."""
    with open(filename, 'r') as f:
        yaml_cfg = edict(yaml.load(f))

    _merge_a_into_b(yaml_cfg, __C)
