"""MVTec Python Library for Evaluation of Tracker Accuracy

This file contains useful methods to read and write the tracker results and
calculated overlaps of theoretical and real trackers.

Copyright (c) MVTec Software GmbH.  All rights reserved.

All use of this software is subject to the terms of the revised BSD
license.  You should have received a copy of this license along
with this source code in a file named "license.txt."
"""
import pandas as pd


def read_csv(csv_file):
    """Reads a csv-file given by csv_file and returns its contents as pandas
    DataFrame.

    Args:
        csv_file(str): Path to csv-file.
    Return:
        pandas.DataFrame: DataFrame containing csv-contents.
    """
    df = pd.read_csv(csv_file, sep=';')
    return df


def get_tracker_ious(overlaps_df, tracker_name):
    """Given a pandas dataframe with overlaps, returns the columns with
    absolute and relative overlaps as lists.

    We assume that the columns for the specific tracker containing the overlaps
    are named 'tracker_<tracker_name>_iou_abs' and
    'tracker_<tracker_name>_iou_rel'.

    Args:
        overlaps_df(pd.DataFrame): DataFrame containing overlaps, as e.g. read
                                   by read_csv.
        tracker_name(str):         Name of the tracker, e.g. 'kcf'.
    Return:
        List(str): Absolute overlaps (IoUs) of the tracker.
        List(str): Relative overlaps (IoUs) of the tracker.
    """
    abs_overlaps = overlaps_df['tracker_' + tracker_name + '_iou_abs'].tolist()
    rel_overlaps = overlaps_df['tracker_' + tracker_name + '_iou_rel'].tolist()
    return abs_overlaps, rel_overlaps


def get_tracker_box(tracker_df, tracker_name, img_idx):
    """Given a pandas dataframe with tracker boxes, returns the box-coordinates
    of a tracker in a frame as list.

    Args:
        tracker_df(pd.DataFrame): DataFrame containing tracker box-params per
                                  tracker and frame.
        tracker_name(str):        Name of the tracker column as in csv,
                                  e.g. 'tracker_dsst'.
        img_idx(int):             Index of the frame.
    Return:
        List(float): box-parameters of the tracker in frame number img_idx.
    """
    # Get values from data-frame as one string
    box_params_str = tracker_df[tracker_name].tolist()[img_idx]
    # Remove the brackets '[', ']'
    box_params_str = box_params_str.replace('[', '')
    box_params_str = box_params_str.replace(']', '')
    # Convert comma-seperated values to list
    box_params = box_params_str.split(',')
    box_params = [float(i) for i in box_params]

    return box_params


def get_theoretical_tracker_ious(overlaps_df, tracker_name):
    """Given a pandas dataframe with overlaps, returns the columns with
    absolute overlaps of a theoretical tracker as list.

    We assume that the columns for the specific tracker containing the overlaps
    are named 'theoretical_<tracker_name>'.

    Args:
        overlaps_df(pd.DataFrame): DataFrame containing overlaps, as e.g. read
                                   by read_csv.
        tracker_name(str):         Name of the tracker, e.g. 'no_scale'.
    Return:
        List(str): Absolute overlaps (IoUs) of the theoretical tracker.
    """
    overlaps = overlaps_df['theoretical_' + tracker_name].tolist()
    return overlaps


def add_tracker_boxes_to_csv(csv_file,
                             tracker_name,
                             boxes,
                             output_csv_file=None):
    """Add a column with tracker output box parameters to a csv-file.

    We name the column for the specific tracker 'tracker_<tracker_name>'.

    Args:
        csv_file(str): Path of the csv-file.
        tracker_name(str): Name of the tracker, e.g. 'awesome'. The column name
                           will then be 'tracker_awesome'. If the column name
                           already exists in this csv-file it will be
                           overwritten.
        boxes(list(list(float))): A list of lists of float-values. Each entry
                                  in the outer list corresponds to the
                                  image-idx of the scene. The inner lists
                                  contain the 5 params of the tracker output
                                  box [center_x, center_y, width, height,
                                  angle(deg)]. The angle should be between -45
                                  and 45 degrees.
        output_csv_file(str): Path of the output csv-file. If not specified,
                              csv-file will be overwritten.

    Author: Patrick Follmann, MVTec Software GmbH, 2017.
    """
    # Read csv-file
    tracker_df = read_csv(csv_file)

    # Check consistency - number of frames
    if len(boxes) != len(tracker_df.index):
        raise Exception("Number of Boxes not consistent with other trackers in"
                        "csv-file: " + csv_file)
    # Convert float values to string and format to 3 decimal places
    boxes_str = []
    for idx in range(len(boxes)):
        cur_str = '['
        for val_idx in range(len(boxes[idx])):
            cur_str += '{:.3f}'.format(boxes[idx][val_idx])
            if val_idx < len(boxes[idx])-1:
                cur_str += ','
        cur_str += ']'
        boxes_str.append(cur_str)

    # Create series
    boxes_se = pd.Series(boxes_str)
    # Append to DataFrame
    tracker_df['tracker_' + tracker_name] = boxes_se
    # Overwrite csv_file if output_csv_file not specified
    if output_csv_file is None:
        output_csv_file = csv_file
    # Write out
    tracker_df.to_csv(output_csv_file, sep=';', index=False)

    return


def add_tracker_ious_to_csv(csv_file,
                            tracker_name,
                            ious_abs,
                            ious_rel,
                            output_csv_file=None):
    """Add columns with absolute and relative tracker ious to a csv-file.

    We name the column for the specific tracker 'tracker_<tracker_name>'.

    Args:
        csv_file(str): Path of the csv-file.
        tracker_name(str): Name of the tracker, e.g. 'awesome'. The column name
                           will then be 'tracker_awesome'. If the column name
                           already exists in this csv-file it will be
                           overwritten.
        ious_abs(list(float)): A list of the absolute IoU values for the
                               tracker output box with the groundtruth
                               segmentation for each frame.
        ious_abs(list(float)): A list of the relative IoU values for the
                               tracker output box with the theoretical tracker
                               box for each frame. Note that the theoretical
                               tracker has to be chosen according to the
                               abilities of the tracker (axis-aligned
                               (no scale), axis-aligned or box_rot, see our
                               paper for details).
        output_csv_file(str): Path of the output csv-file. If not specified,
                              csv-file will be overwritten.

    Author: Patrick Follmann, MVTec Software GmbH, 2017.
    """
    # Read csv-file
    overlaps_df = read_csv(csv_file)

    # Check consistency - number of frames
    if len(ious_abs) != len(overlaps_df.index) or \
       len(ious_rel) != len(overlaps_df.index):
        raise Exception("Number of IoU-values not consistent with other"
                        "trackers in csv-file: " + csv_file)
    # Create series (convert format to 3 decimal places)
    ious_abs_se = pd.Series(['{:.3f}'.format(x) for x in ious_abs])
    ious_rel_se = pd.Series(['{:.3f}'.format(x) for x in ious_rel])
    # Append to DataFrame
    overlaps_df['tracker_' + tracker_name + '_iou_abs'] = ious_abs_se
    overlaps_df['tracker_' + tracker_name + '_iou_rel'] = ious_rel_se
    # Overwrite csv_file if output_csv_file not specified
    if output_csv_file is None:
        output_csv_file = csv_file
    # Write out
    overlaps_df.to_csv(output_csv_file, sep=';', index=False)

    return
