"""MVTec Python Visualization Library for Evaluation of Tracker Accuracy

This File contains useful plotting methods to visualize the absolute and
relative IoU of different trackers.

Copyright (c) MVTec Software GmbH.  All rights reserved.
#
All use of this software is subject to the terms of the revised BSD
license.  You should have received a copy of this license along
with this source code in a file named "license.txt."
"""
import matplotlib.pyplot as plt
import numpy as np
from PIL import Image, ImageDraw, ImageChops
import matplotlib.colors as mplc
import mvtec_csv
import mvtec_util
import os

# set the default size and resolution of a figure
plt.rcParams['figure.figsize'] = (15, 10)
plt.rcParams['figure.dpi'] = 110
plt.rcParams['font.size'] = 8

# define a color-cycle with 32 colors for color-type "cycle"
colormap = plt.cm.gist_ncar
cycle_colors = [colormap(i) for i in np.linspace(0, 0.9, 32)]
color_count = 0

fmt_str = "{0:23} {1}"


def plot_lines(data_x,
               data_y,
               label_x=None,
               label_y=None,
               title=None,
               plot_type='-',
               color='red',
               figure=None,
               linewidth=1):
    """Plots a line through the points given by (data_x, data_y).

    Optionally adds a title and axis-names.

    Args:
        data_x(float): x-coordinates of points.
        data_y(float): y-coordinates of points.
        label_x(str): label of the x-axis
        label_y(str): label of the y-asis
        title(str): title of the plot.
        plot_type(str): type of the line or points to plot.
        color(str): color of the line to plot. If color is set to 'cycle' the
               color will be cycling through 32 pre-defined colors each time
               the method is called.
        figure: figure-handle to draw within. If not specified,
                a new figure will be generated.
        linewidth(float): width of the plot lines

    author: Patrick Follmann, MVTec Software GmbH 2016.
    """
    ax = figure.add_subplot(111)
    ax.plot(data_x, data_y, plot_type, color=color, linewidth=linewidth)

    # Always set title newly if present!
    if title is not None:
        ax.set_title(title, y=1.04)

    # add axes-labels and set axes-scale
    if label_y is not None:
        ax.set_ylabel(label_y)
        ax.yaxis.label.set_fontsize(20)
    if label_x is not None:
        ax.set_xlabel(label_x)

    return figure


def add_legend(figure,
               legend,
               location='center left',
               bbox_to_anchor=(1, 0.5)):
    """Add a legend to a figure.

    Args:
        figure: the figure where the legend should be added.
                Has to contain a subplot 111.
        legend(str): the text to appear in the legend.
                Has to have the same length as drawings in the plot.
        location(int): position of the text within the legend box.
        bbox_to_anchor: position of the legend box with respect to the plot.

    Author: Patrick Follmann, MVTec Software GmbH 2016.
    """
    # Resize all x-axes (bottom and top)
    for ax in figure.get_axes():
        box = ax.get_position()
        ax.set_position([box.x0, box.y0, 0.6, box.height])

    # Add legend
    ax = figure.add_subplot(111)
    ax.legend(legend, loc=location, bbox_to_anchor=bbox_to_anchor)

    plt.draw()
    return


def add_point_marker(figure,
                     data_x,
                     data_y,
                     plot_type='x',
                     color='red',
                     markersize=5):
    """Add point-markers to a figure.

    Args:
        figure: the figure where the legend should be added.
                Has to contain a subplot 111.
        data_x(float): the x-coordinates of the markers.
        data_y(float): the y-coordinates of the markers.
        plot_type(str): the type of the marker.
        color(str): the color of the marker (see plot_lines)
        markersize(float): the size of the marker.

    Author: Patrick Follmann, MVTec Software GmbH 2016.
    """
    ax = figure.add_subplot(111)
    ax.plot(data_x, data_y, plot_type, color=color, markersize=markersize)

    plt.draw()
    return


def change_ylim(figure,
                new_ylim):
    """Change the y-lim of a figure.

    Args:
        figure: the figure where the ylim should be changed.
        new_ylim: the new value-range of the y-axis.
                  (given as list [ymin, ymax])

    Author: Patrick Follmann, MVTec Software GmbH 2016.
    """
    for ax in figure.get_axes():
        ax.set_ylim(new_ylim)

    return


def change_xlim(figure,
                new_xlim):
    """Change the x-lim of a figure.

    Args:
        figure: the figure where the xlim should be changed.
        new_xlim: the new value-range of the x-axis.
                  (given as list [xmin, xmax])

    Author: Patrick Follmann, MVTec Software GmbH 2017.
    """
    for ax in figure.get_axes():
        ax.set_xlim(new_xlim)

    return


def plot_overlaps(overlaps,
                  figure,
                  sequence_name,
                  overlap_type='abs',
                  color=None,
                  linewidth=1,
                  plot_no_title=False):
    """Create an overlap-plot for the evaluation-app.

    Args:
        overlaps(List(float)): Data to plot on y-axis, either relative or
                               absolute overlaps of the tracker.
        figure(matplotlib.figure.Figure): the figure to be used for plotting.
        sequence_name(str: name of the sequence (e.g. 'bear')
        overlap_type(str): Indicating if the plot is for absolute ('abs') or
                           relative ('rel') overlaps
        color(str): color of the line to plot. If color is set to 'cycle' the
               color will be cycling through 32 pre-defined colors each time
               the method is called.
        linewidth(int): width of the plot lines
        plot_no_title(bool): flag which sets if the title is plot or not

    Author: Patrick Follmann, MVTec Software GmbH 2017.
    """
    if overlap_type == 'abs':
        title = 'Sequence: ' + sequence_name
        label_y = '$\Phi_{IoU}$'
    elif overlap_type == 'rel':
        title = 'Sequence: ' + sequence_name
        label_y = '$\Phi_{rIoU}$'
    else:
        print fmt_str.format('plot_overlaps', 'Unknown overlap_type: {}, '
                                              'please choose from [\'rel\', '
                                              '\'abs\']'.format(overlap_type))
        return

    # Don't plot title if you don't wish so
    if plot_no_title:
        title = None

    figure = plot_lines(range(0, len(overlaps), 1),
                        overlaps,
                        label_x='Image Index',
                        label_y=label_y,
                        title=title,
                        color=color,
                        figure=figure,
                        linewidth=linewidth)
    change_ylim(figure, [0, 1])
    change_xlim(figure, [1, len(overlaps)])

    return


def plot_idx_line(line_handle,
                  cur_idx,
                  figure):
    """Plot a vertical red line into the figure at a given index.

    Args:
        line_handle(ax.plot handle): handle to the ax.plot line
        cur_idx(int): Position on x-axis where the line should be plotted.
        figure(matplotlib.figure.Figure): the figure to be used for plotting.

    Author: Patrick Follmann, MVTec Software GmbH 2017.
    """
    ax = figure.add_subplot(111)
    if line_handle is None:
        line_handle, = ax.plot([cur_idx, cur_idx], [0, 1], '-', color='red')
    else:
        line_handle.set_xdata([cur_idx, cur_idx])
        ax.draw_artist(line_handle)

    return line_handle


def plot_helper_lines(yticks,
                      xlim,
                      figure):
    """Plot gray dashed horizontal lines into the figure at positions yticks.

    Args:
        yticks(list(float)): Positions at y-axis where horizontal lines should
                             be added.
        xlim(int): maximum value of x-axis.
        figure(matplotlib.figure.Figure): the figure to be used for plotting.

    Author: Patrick Follmann, MVTec Software GmbH 2017.
    """
    ax = figure.add_subplot(111)
    for tick in yticks:
        ax.plot([0, xlim], [tick, tick], '--', color='gray')

    return


def draw_box(image,
             tracker_df,
             img_idx,
             tracker_name,
             color,
             alpha=90):
    """Draw the result-box for a tracker into a PIL image.

    Args:
        image(Image): PIL image where the box should be drawn into.
        tracker_df(pandas.DataFrame): data-frame that contains box-params for
                                      all trackers.
        img_idx(int): Index of the image in the sequence (frame number).
        tracker_name(str): Name of the tracker, e.g. 'kcf'.
        color(hex): Color to be used to draw the box.
        alpha(int): Alpha value controlling the opacity of the box (0 < alpha
                   <=255).
    Return:
        Image: input image with drawn box.

    Author: Patrick Follmann, MVTec Software GmbH 2017.
    """
    # Check if the tracker is available
    if tracker_name not in tracker_df.columns.tolist():
        print fmt_str.format('draw_box',
                             'Unknown tracker_name: {}'.format(tracker_name))
        return

    # Get box-coordinates from data-frame
    tracker_box_params = mvtec_csv.get_tracker_box(tracker_df, tracker_name,
                                                   img_idx)
    # transform box-coordinates to corner-points
    box_corners = mvtec_util.convert_box2polygon(tracker_box_params)

    # Draw box onto the image
    draw = ImageDraw.Draw(image, 'RGBA')
    # Convert color from hex to rgb and append the alpha channel
    rgb_color = mplc.hex2color(color)
    draw_color = (int(rgb_color[0] * 255), int(rgb_color[1] * 255),
                  int(rgb_color[2] * 255), alpha)
    draw.polygon(box_corners, outline=color, fill=draw_color)
    del draw

    return image


def draw_segmentation(annotations_dir,
                      image,
                      annot_name,
                      color='#ff0000',
                      alpha=60):
    """Highlight the segmentation region in a PIL image.

    Args:
        annotations_dir(str): directory of the annotations (segmentations)
        image(Image): PIL image where the segmentation should be drawn into.
        annot_name(str): filename of the corresponding annotation.
        color(hex): Color to be used to draw the box.
        alpha(int): Alpha value controlling the opacity of the box (0 < alpha
                    <=255).
    Return:
        Image: input image with drawn segmentation.

    Author: Patrick Follmann, MVTec Software GmbH 2017.
    """
    # Get the segmentation from file
    segmentation_file = os.path.join(annotations_dir, annot_name)
    # Check if segmentation is available
    if not os.path.isfile(segmentation_file):
        print fmt_str.format('draw_segmentation',
                             'File does not exist: {}'
                             .format(segmentation_file))
        return image
    # Read in the segmentation and multiply with alpha-value to create a mask
    segmentation = Image.open(segmentation_file).convert("L")
    alpha_image = Image.new("L", image.size, alpha)
    mask = ImageChops.multiply(alpha_image, segmentation)

    # Convert color from hex to rgb and append
    rgb_color = mplc.hex2color(color)
    draw_color = (int(rgb_color[0] * 255), int(rgb_color[1] * 255),
                  int(rgb_color[2] * 255))
    # Create new color-image to blend over:
    blend_image = Image.new("RGBA", image.size, draw_color)
    # Alpha-channel is interpreted as mask
    blend_image.putalpha(mask)
    image.paste(blend_image, (0, 0), blend_image)

    return image
