#!/usr/bin/env python

# --------------------------------------------------------
# App to visualize tracker results
# Copyright (c) MVTec Software GmbH.  All rights reserved.
#
# All use of this software is subject to the terms of the revised BSD
# license.  You should have received a copy of this license along
# with this source code in a file named "license.txt."
# --------------------------------------------------------

import matplotlib_tkagg_backend  # NOQA
from matplotlib.figure import Figure
import os
import Tkinter
import tkFont
import glob
import time
from PIL import Image, ImageTk
from multiprocessing import Queue, Process
import ttk
import mvtec_csv
import mvtec_plot
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
from matplotlib.backends.backend_tkagg import NavigationToolbar2TkAgg
import matplotlib.pyplot as plt
import matplotlib.colors as mplc
import numpy as np


class Checkbar(Tkinter.Frame):
    def __init__(self, parent=None, picks=[], command=None, colors=None,
                 texts=None):
        Tkinter.Frame.__init__(self, parent)
        self.vars = []
        self.chks = []
        self.picks = picks
        self.grid()
        if texts is None:
            texts = picks
        for idx in range(len(picks)):
            var = Tkinter.IntVar()
            chk = Tkinter.Checkbutton(self, text=texts[idx], variable=var,
                                      command=command)
            chk.grid(row=idx, column=0, sticky='W')
            self.vars.append(var)
            self.chks.append(chk)
            canv = Tkinter.Canvas(self, width=20, height=10)
            canv.grid(row=idx, column=1, sticky='W')
            canv.create_line(0, 5, 20, 5, fill=colors[idx], width=3)

    def state(self):
        mapping = map((lambda var: var.get()), self.vars)
        return [self.picks[idx] for idx in range(len(self.picks))
                if mapping[idx] > 0]


def is_number(s):
    try:
        float(s)
        return True
    except ValueError:
        return False


class TrackerEvaluationApp():

    def __init__(self,
                 cfg):

        # Setup initial parameters
        self.cfg = cfg
        self.images_base = os.path.join(cfg.BASE_DIR, cfg.IMAGE_DIR)
        self.annotations_base = os.path.join(cfg.BASE_DIR, cfg.ANNOT_DIR)
        os.chdir(self.images_base)
        self.sequence_idx = 0
        self.set_sequence_folders()

        # Initialize Stream
        self.streaming = False
        self.pause = False
        self.stop = False
        # Initialize queue and image reading process
        self.queue = Queue()
        self.process = None

        # Initialize GUI (thread)
        self.root = Tkinter.Tk()

        # Use a grid to position widgets
        self.root.grid()

        # Change default font size
        default_font = tkFont.nametofont("TkDefaultFont")
        default_font.configure(size=8)
        self.root.option_add("*Font", default_font)

        # #######################  STREAM IMAGE ###############################
        # Initialize Sequence Label
        self.seq_label = Tkinter.StringVar()
        seq_label = Tkinter.Label(master=self.root,
                                  textvariable=self.seq_label,
                                  font=('TkDefaultFont', 14))
        seq_label.grid(row=0, column=0, columnspan=7, sticky='N')

        # Initialize Image Label
        self.image_label = Tkinter.Label(master=self.root)
        self.image_label.grid(row=1, column=0, rowspan=6, columnspan=7,
                              sticky='N')

        # ########################  EVALUATION PLOTS ##########################
        # Initialize Canvas to Plot results
        self.plot_size = self.cfg.PLOT_SIZE
        self.stream_btn_row = 7  # has to be defined before the next call
        self.init_figures()

        # #######################  STREAM BUTTONS  ############################
        btn_lbl_width = 13
        stream_btn_row = 7

        # Setup Play Button
        self.play_button = Tkinter.Button(master=self.root, text='Play',
                                          width=btn_lbl_width,
                                          command=lambda:
                                          self.on_play_button_click())
        self.play_button.grid(row=stream_btn_row, column=2, sticky='NEW')

        # Setup Pause button
        self.pause_button = Tkinter.Button(self.root, text='Pause',
                                           width=btn_lbl_width,
                                           command=self.on_pause_button_click)
        self.pause_button.grid(column=3, row=stream_btn_row, sticky='NEW')

        # Setup Stop button
        stop_button = Tkinter.Button(self.root, text='Stop',
                                     width=btn_lbl_width,
                                     command=self.on_stop_button_click)
        stop_button.grid(column=4, row=stream_btn_row, sticky='NEW')

        # Setup the Next and Previous Image buttons
        previous_img_button = Tkinter.Button(self.root, text="Previous Image",
                                             width=btn_lbl_width,
                                             command=self.
                                             on_previous_img_button_click)
        previous_img_button.grid(column=1, row=stream_btn_row, stick='NEW')

        next_img_button = Tkinter.Button(self.root, text="Next Image",
                                         width=btn_lbl_width,
                                         command=self.on_next_img_button_click)
        next_img_button.grid(column=5, row=stream_btn_row, sticky='NEW')

        # Setup the Next and Previous Sequence buttons
        previous_seq_button = Tkinter.Button(self.root, width=btn_lbl_width,
                                             text="Previous Sequence",
                                             command=self.
                                             on_previous_seq_button_click)
        previous_seq_button.grid(column=0, row=stream_btn_row, stick='NEW')

        next_seq_button = Tkinter.Button(self.root, text="Next Sequence",
                                         width=btn_lbl_width,
                                         command=self.on_next_seq_button_click)
        next_seq_button.grid(column=6, row=stream_btn_row, sticky='NEW')

        # ######################## STREAM INFO ################################
        # Setup the labels displaying the current image name and index and
        # the min and max index
        self.img_label = Tkinter.StringVar()
        label = Tkinter.Label(self.root, textvariable=self.img_label,
                              width=btn_lbl_width,
                              fg="black", bg="grey")
        label.grid(column=4, row=stream_btn_row+2, sticky='NEW')

        self.img_idx_label_low = Tkinter.StringVar()
        lowlabel = Tkinter.Label(self.root, width=btn_lbl_width,
                                 textvariable=self.img_idx_label_low,
                                 fg="black", bg="grey")
        lowlabel.grid(column=2, row=stream_btn_row+1, sticky='NEW')

        self.img_idx_label = Tkinter.StringVar()
        idxlabel = Tkinter.Label(self.root, width=btn_lbl_width,
                                 textvariable=self.img_idx_label,
                                 fg="black", bg="grey")
        idxlabel.grid(column=3, row=stream_btn_row+1, sticky='NEW')

        self.img_idx_label_high = Tkinter.StringVar()
        highlabel = Tkinter.Label(self.root, width=btn_lbl_width,
                                  textvariable=self.img_idx_label_high,
                                  fg="black", bg="grey")
        highlabel.grid(column=4, row=stream_btn_row+1, sticky='NEW')

        # Add an input field where the user can manually adapt the
        # current Index
        self.input_idx_entry = Tkinter.IntVar()
        idx_entry = Tkinter.Entry(self.root, width=btn_lbl_width,
                                  textvariable=self.input_idx_entry,
                                  fg="black", bg="white")
        idx_entry.grid(column=3, row=stream_btn_row+2, sticky='NEW')
        idx_entry.focus()
        idx_entry.delete(0, 'end')
        idx_entry.insert(0, 0)
        # Bind the entry box to the Return key
        self.root.bind('<Return>', self.on_return)
        self.idx_entry = idx_entry

        # #####################  SETTINGS COLUMN  #############################
        set_column = 7
        # We use another width for the settings
        set_width = 19
        # Setup Quit Button
        quit_button = Tkinter.Button(master=self.root, text='Quit',
                                     width=set_width + 3,
                                     command=lambda: self.quit())
        quit_button.grid(row=0, column=set_column, sticky='NEW')

        # Setup Combobox to select sequence
        self.seq_combobox = ttk.Combobox(master=self.root, width=set_width,
                                         state='readonly',
                                         font=('TkDefaultFont', 12))
        self.seq_combobox['values'] = self.sequence_folders
        self.seq_combobox.current(0)
        self.seq_combobox.bind("<<ComboboxSelected>>", self.select_sequence)
        self.seq_combobox.grid(row=1, column=set_column, sticky='NEW')

        # Checkboxes to show segmentation and selection of trackers for
        # display and evaluation
        show_segmentation = ['show segmentation']
        self.show_segmentation = Checkbar(self.root,
                                          show_segmentation,
                                          self.update_selection,
                                          ['#ff0000'])
        self.show_segmentation.grid(row=2, column=set_column, sticky='NEW')
        self.theoretical_trackers = self.cfg.THEORETICAL_TRACKERS
        self.trackers = self.cfg.REAL_TRACKERS
        colormap = plt.cm.gist_ncar
        self.tracker_colors = [mplc.rgb2hex(colormap(i)[:3]) for i in
                               np.linspace(0, 0.9,
                                           len(self.theoretical_trackers) +
                                           len(self.trackers))]
        label_theoretical_trackers = Tkinter.Label(self.root,
                                                   text='Theoretical Trackers',
                                                   font=('TkDefaultFont', 12))
        label_theoretical_trackers.grid(row=3, column=set_column, sticky='NEW')
        # Show segmentations per default
        self.show_segmentation.chks[0].select()
        self.selected_theoretical_trackers = Checkbar(
                        self.root,
                        self.theoretical_trackers,
                        self.update_selection,
                        self.tracker_colors[0:len(self.theoretical_trackers)],
                        self.theoretical_trackers,
                                             )
        self.selected_theoretical_trackers.grid(row=4, column=set_column,
                                                sticky='NW')
        label_trackers = Tkinter.Label(self.root, text='Trackers',
                                       font=('TkDefaultFont', 12))
        label_trackers.grid(row=5, column=set_column, sticky='NEW')
        self.selected_trackers = Checkbar(
                    self.root,
                    self.trackers,
                    self.update_selection,
                    colors=self.tracker_colors[len(self.theoretical_trackers):]
                                 )
        self.selected_trackers.grid(row=6, column=set_column, sticky='NEW')

        # #####################################################################

        # Enable the resizing of the whole window
        self.root.grid_columnconfigure(0, weight=1)
        self.root.grid_rowconfigure(0, weight=1)

        # Set title of App
        self.root.title('Accuracy of Trackers Evaluation App')

        # Go to current sequence and show first image
        self.change_sequence(self.sequence_idx)
        self.set_image()
        print 'Initialization Done'

    def on_return(self, event):
        # When return is pressed, we get the entry field entry and
        # update the image index
        try:
            idx = self.input_idx_entry.get()
            if is_number(idx):
                if idx >= 0 and idx < self.num_images:
                    self.img_idx = idx
                    self.set_image()
                    self.update_plots()
                    self.img_idx_label.set(self.img_idx)
                    self.idx_entry.focus()
                    self.img_label.set(self.img_files[self.img_idx])
                else:
                    self.idx_entry.delete(0, 'end')
            else:
                self.idx_entry.delete(0, 'end')
        except ValueError:
            self.idx_entry.delete(0, 'end')

    def on_next_img_button_click(self):
        if not self.streaming:
            # Increment the image idx and display the new results
            if self.img_idx < self.num_images - 1:
                self.img_idx = self.img_idx + 1
                # Display new image with results
                self.set_image()
                self.update_plots()
                self.idx_entry.delete(0, 'end')
                self.idx_entry.insert(0, str(self.img_idx))
                self.img_idx_label.set(self.img_idx)
                self.img_label.set(self.img_files[self.img_idx])

    def on_previous_img_button_click(self):
        if not self.streaming:
            if self.img_idx > 0:
                self.img_idx = self.img_idx - 1
                # Display new image with results
                self.set_image()
                self.update_plots()
                self.idx_entry.delete(0, 'end')
                self.idx_entry.insert(0, str(self.img_idx))
                self.img_idx_label.set(self.img_idx)
                self.img_label.set(self.img_files[self.img_idx])

    def on_next_seq_button_click(self):
        if self.streaming or self.pause:
            self.stop_stream()
        # Increment the sequence idx and display
        if self.sequence_idx < self.num_sequences - 1:
            self.change_sequence(self.sequence_idx + 1)
            self.set_image()

    def on_previous_seq_button_click(self):
        if self.streaming or self.pause:
            self.stop_stream()
        # Decrement the sequence idx and display
        if self.sequence_idx > 0:
            self.change_sequence(self.sequence_idx - 1)
            self.set_image()

    def select_sequence(self, event):
        if self.streaming or self.pause:
            self.stop_stream()
        # Get idx of selected sequence in Combobox and change to it
        idx = self.seq_combobox.current()
        self.seq_combobox.selection_clear()
        self.change_sequence(idx)
        self.set_image()

    def on_play_button_click(self):
        self.streaming = True
        self.stop = False
        self.play_button.config(relief=Tkinter.SUNKEN,
                                state=Tkinter.DISABLED)
        self.process = Process(target=self.image_reading, args=())
        self.process.start()
        if self.pause:
            self.pause = False
            self.pause_button.config(relief=Tkinter.RAISED,
                                     state=Tkinter.ACTIVE)
        # setup the update callback
        self.stream()

    def on_pause_button_click(self):
        if self.streaming:
            self.streaming = False
            self.pause = True
            self.pause_button.config(relief=Tkinter.SUNKEN,
                                     state=Tkinter.DISABLED)
            self.play_button.config(relief=Tkinter.RAISED,
                                    state=Tkinter.ACTIVE)
            if self.process is not None:
                self.process.join(1)
                if self.process.is_alive:
                    self.process.terminate()
            self.queue.close()
            self.queue = Queue()

    def on_stop_button_click(self):
        self.img_idx = 0
        self.stop_stream()
        self.set_image()
        self.update_plots()

    def stop_stream(self):
        if self.streaming:
            self.streaming = False
        # Raise Play-Button only if we are not at the end of the sequence
        if self.img_idx != self.num_images - 1:
            self.play_button.config(relief=Tkinter.RAISED,
                                    state=Tkinter.ACTIVE)
        if self.pause:
            self.pause = False
            self.pause_button.config(relief=Tkinter.RAISED,
                                     state=Tkinter.ACTIVE)
        self.stop = True
        if self.process is not None:
            self.process.join(1)
            if self.process.is_alive:
                self.process.terminate()
        self.queue.close()
        self.queue = Queue()

    def set_sequence_folders(self):
        # Get sorted list of sequences
        self.sequence_folders = [d for d in os.listdir(self.images_base) if
                                 os.path.isdir(os.path.join(self.images_base,
                                                            d))]
        self.sequence_folders.sort()
        self.num_sequences = len(self.sequence_folders)
        self.sequence_idx = 0

    def change_sequence(self, sequence_idx):
        self.sequence_idx = sequence_idx
        self.sequence_name = self.sequence_folders[self.sequence_idx]
        self.annotations_dir = os.path.join(self.annotations_base,
                                            self.sequence_name)
        self.seq_label.set('Sequence: {}'.format(self.sequence_name))
        # Get list of available annotations (.png-files)
        os.chdir(self.annotations_dir)
        annot_files = glob.glob('*.png')
        # sort by image-name (in this case only integers)
        annot_files = [annot_file.split('.')[0] for annot_file in annot_files]
        annot_files.sort(key=int)
        annot_files = [annot_file + '.png' for annot_file in annot_files]
        self.annot_files = annot_files
        # Get list of available .jpg-files in this folder
        os.chdir(os.path.join(self.images_base, self.sequence_name))
        img_files = glob.glob('*.jpg')
        # sort by image-name (in this case only integers)
        img_files = [img_file.split('.')[0] for img_file in img_files]
        img_files.sort(key=int)
        img_files = [img_file + '.jpg' for img_file in img_files]
        self.img_files = img_files
        self.num_images = len(self.img_files)
        self.img_idx = 0
        # Update labels
        self.img_label.set(self.img_files[self.img_idx])
        self.idx_entry.delete(0, 'end')
        self.idx_entry.insert(0, '0')
        self.img_idx_label_high.set(self.num_images - 1)
        self.img_idx_label_low.set(0)
        self.img_idx_label.set(0)
        self.seq_combobox.current(self.sequence_idx)
        # Clear figures
        self.clear_figures()
        # Get csv-info
        self.get_seq_info()
        self.evaluate_sequence()
        # Raise Play-Button
        self.play_button.config(relief=Tkinter.RAISED,
                                state=Tkinter.ACTIVE)

    def load_image(self, index):
        # Load and display the current image
        img_name = self.img_files[index]
        pil_image = Image.open(img_name)
        # Draw segmentation
        if len(self.show_segmentation.state()) > 0:
            annot_name = self.annot_files[index]
            pil_image = mvtec_plot.draw_segmentation(self.annotations_dir,
                                                     pil_image,
                                                     annot_name)
        # Draw tracker boxes
        # theoretical trackers
        for tracker in self.selected_theoretical_trackers.state():
            idx = self.theoretical_trackers.index(tracker)
            pil_image = mvtec_plot.draw_box(
                    pil_image,
                    self.trackers_df,
                    index,
                    'theoretical_' + tracker,
                    self.tracker_colors[idx]
                        )
        # real trackers
        for tracker in self.selected_trackers.state():
            idx = self.trackers.index(tracker)
            pil_image = mvtec_plot.draw_box(
                    pil_image,
                    self.trackers_df,
                    index,
                    'tracker_' + tracker,
                    self.tracker_colors[len(self.theoretical_trackers) + idx]
                        )
        return pil_image

    def set_image(self):
        # Load and display the current image
        pil_image = self.load_image(self.img_idx)
        pil_image.thumbnail(self.cfg.STREAM_SIZE)
        self.photo_image = ImageTk.PhotoImage(image=pil_image)
        self.image_label.configure(image=self.photo_image)
        self.img_label.set(self.img_files[self.img_idx])
        self.idx_entry.delete(0, 'end')
        self.idx_entry.insert(0, str(self.img_idx))
        self.img_idx_label.set(self.img_idx)
        self.root.update()

    def quit(self):
        if self.process is not None and self.process.is_alive():
            self.process.terminate()
            self.process.join()
        self.root.destroy()

    def update_image(self):
        try:
            frame, cur_idx = self.queue.get()
            if cur_idx == -1 or not self.streaming:
                self.stop_stream()
                return
            self.img_idx = cur_idx
            # Update image on image_label
            self.photo_image = ImageTk.PhotoImage(image=frame)
            self.image_label.configure(image=self.photo_image)
            # Update evaluation plot - this slows down the movie a lot!
            self.update_plots()
            self.img_label.set(self.img_files[self.img_idx])
            self.idx_entry.delete(0, 'end')
            self.idx_entry.insert(0, str(self.img_idx))
            self.img_idx_label.set(cur_idx)
            self.root.update()
        except KeyboardInterrupt:
            print 'ignore CTRL-C from worker'

    def stream(self):
        if self.streaming:
            self.update_image()
            self.root.after(0, func=lambda: self.stream())

    def image_reading(self):
        cur_idx = self.img_idx
        while self.streaming and not self.stop:
            if cur_idx < self.num_images:
                # Load the image
                img = self.load_image(cur_idx)
                img.thumbnail(self.cfg.STREAM_SIZE)
                self.queue.put((img, cur_idx))
                cur_idx = cur_idx + 1
                time.sleep(0.01)
            else:
                self.queue.put((None, -1))
                break

    def get_seq_info(self):
        overlaps_csv = self.cfg.OVERLAP_CSV
        self.overlaps_df = mvtec_csv.read_csv(overlaps_csv)
        tracker_csv = self.cfg.TRACKER_CSV
        self.trackers_df = mvtec_csv.read_csv(tracker_csv)

    def evaluate_sequence(self):
        self.clear_figures()
        # Check if we need an absolute IoU plot
        if len(self.selected_trackers.state()) + \
                len(self.selected_theoretical_trackers.state()) > 0:
            # Plot selected theoretical tracker bb_overlaps
            for tracker in self.selected_theoretical_trackers.state():
                abs_bb_overlaps = mvtec_csv.get_theoretical_tracker_ious(
                                self.overlaps_df, tracker
                                  )
                idx = self.theoretical_trackers.index(tracker)
                mvtec_plot.plot_overlaps(abs_bb_overlaps,
                                         self.figure_abs,
                                         self.sequence_name,
                                         overlap_type='abs',
                                         color=self.tracker_colors[idx])
            # Check if we need a relative IoU plot
            if len(self.selected_trackers.state()) > 0:
                # Plot selected tracker bb_overlaps
                for tracker in self.selected_trackers.state():
                    abs_bb_overlaps, rel_bb_overlaps = \
                        mvtec_csv.get_tracker_ious(self.overlaps_df, tracker)
                    idx = self.trackers.index(tracker)
                    # absolute IoU
                    mvtec_plot.plot_overlaps(
                        abs_bb_overlaps,
                        self.figure_abs,
                        self.sequence_name,
                        overlap_type='abs',
                        color=self.tracker_colors[
                            len(self.theoretical_trackers) + idx
                              ]
                    )
                    # relative IoU
                    mvtec_plot.plot_overlaps(
                        rel_bb_overlaps,
                        self.figure_rel,
                        self.sequence_name,
                        overlap_type='rel',
                        color=self.tracker_colors[len(self.
                                                      theoretical_trackers) +
                                                  idx]
                    )
                # Show vertical red line at current index
                self.vert_line_rel = mvtec_plot.plot_idx_line(None,
                                                              self.img_idx,
                                                              self.figure_rel)
                self.eval_rel = True
            else:
                self.eval_rel = False
            # Show vertical red line at current index
            self.vert_line_abs = mvtec_plot.plot_idx_line(None,
                                                          self.img_idx,
                                                          self.figure_abs)
            self.eval_abs = True
        else:
            self.eval_abs = False
            self.eval_rel = False
            self.vert_line_abs = None
            self.vert_line_rel = None
        # Show absolute IoU plot
        self.results_canvas_abs.show()
        # Show relative IoU plot
        self.results_canvas_rel.show()

    def update_plots(self):
        if self.eval_abs:
            self.vert_line_abs = mvtec_plot.plot_idx_line(self.vert_line_abs,
                                                          self.img_idx,
                                                          self.figure_abs)
            self.results_canvas_abs.draw()
        else:
            # Include a break to slow down video-playing
            time.sleep(0.01)
        if self.eval_rel:
            self.vert_line_rel = mvtec_plot.plot_idx_line(self.vert_line_rel,
                                                          self.img_idx,
                                                          self.figure_rel)
            self.results_canvas_rel.draw()
        else:
            # Include a break to slow down video-playing
            time.sleep(0.01)

    def init_figures(self):
        eval_row = self.stream_btn_row + 3
        frame_height = self.plot_size[1] * 100 + 35
        layout_dict = {'pad': 1.5}
        self.eval_frame = Tkinter.Frame(self.root, height=frame_height)
        # Position eval_frame in root
        self.eval_frame.grid(row=eval_row, column=0, columnspan=8,
                             sticky='NEW')
        self.eval_frame.grid_propagate(False)
        # Generate Figure Canvas for absolute and relative plots
        # We place them in Frames to show and hide them later on
        self.frame_canvas_abs = Tkinter.Frame(master=self.eval_frame)
        self.frame_canvas_rel = Tkinter.Frame(master=self.eval_frame)
        self.frame_canvas_abs.grid(row=0, column=0, sticky='W', padx=30)
        self.frame_canvas_rel.grid(row=0, column=1, sticky='W', padx=30)
        # Get default background color to apply it to the figures
        defaultbg = self.root.cget('bg')
        self.figure_abs = Figure(figsize=(self.plot_size[0],
                                          self.plot_size[1]),
                                 dpi=100, facecolor=defaultbg,
                                 tight_layout=layout_dict)
        self.figure_rel = Figure(figsize=(self.plot_size[0],
                                          self.plot_size[1]),
                                 dpi=100, facecolor=defaultbg,
                                 tight_layout=layout_dict)
        self.results_canvas_abs = FigureCanvasTkAgg(
                            self.figure_abs,
                            master=self.frame_canvas_abs
                                  )
        self.results_canvas_rel = FigureCanvasTkAgg(
                            self.figure_rel,
                            master=self.frame_canvas_rel
                                  )
        # Add Navigation Toolbars to be able to save plots
        self.frame_nav_toolbar_abs = Tkinter.Frame(self.eval_frame)
        self.frame_nav_toolbar_rel = Tkinter.Frame(self.eval_frame)
        self.nav_toolbar_abs = NavigationToolbar2TkAgg(
                            self.results_canvas_abs,
                            self.frame_nav_toolbar_abs
                               )
        self.nav_toolbar_rel = NavigationToolbar2TkAgg(
                            self.results_canvas_rel,
                            self.frame_nav_toolbar_rel
                               )
        # Position Toolbar inside the eval_frame
        self.frame_nav_toolbar_abs.grid(row=1, column=0, sticky='W')
        self.frame_nav_toolbar_rel.grid(row=1, column=1, sticky='W', padx=30)
        self.results_canvas_abs.get_tk_widget().pack()
        self.results_canvas_rel.get_tk_widget().pack()

    def clear_figures(self):
        # Clear absolute IoU plot(s)
        for subplot in self.figure_abs.get_axes():
            subplot.clear()
        # Clear relative IoU plot(s)
        for subplot in self.figure_rel.get_axes():
            subplot.clear()

    def show_figures(self):
        self.results_canvas_abs.get_tk_widget().grid()
        self.results_canvas_rel.get_tk_widget().grid()
        self.results_canvas_abs.show()
        self.results_canvas_rel.show()

    def update_selection(self):
        self.set_image()
        self.evaluate_sequence()
