# ################################################################ #
# Evaluating Accuracy Measures - mvtec_util.py                     #
#                                                                  #
# This file contains helper functions, e.g. to convert different   #
# parametrizations into each other.                                #
#                                                                  #
# Copyright (c) MVTec Software GmbH.  All rights reserved.         #
#                                                                  #
# All use of this software is subject to the terms of the revised  #
# BSD license.                                                     #
# You should have received a copy of this license along            #
# with this source code in a file named "license.txt."             #
# ################################################################ #

import math
from math import radians, cos, sin, atan2
from PIL import Image, ImageDraw
import numpy as np
import os

fmt_str = "{0:23} {1}"


def convert_box2polygon(box_params):
    """Convert a 5-param box to a polygon.

    This is the inverse of convert_polygon2box.

    Args:
        box_params(list(float)): Parameters of the bounding box:
                                [center_x, center_y, width, height, angle(deg)]

    Return:
        list(float): Polygon coordinates of 4 corner points:
                     [x1, y1, x2, y2, ..., x4, y4]

    Author: Patrick Follmann, MVTec Software GmbH, 2017
    """
    # Get coordinates to work with them
    center_x = box_params[0]
    center_y = box_params[1]
    width = box_params[2]
    height = box_params[3]
    phi = radians(box_params[4])

    # Calculate corner-points
    co = cos(phi)
    si = sin(phi)
    l1 = (width - 1) / 2.0
    l2 = (height - 1) / 2.0
    co1 = co * l1
    co2 = co * l2
    si1 = si * l1
    si2 = si * l2

    y1 = co1 - si2 + center_y
    x1 = -si1 - co2 + center_x
    y2 = -co1 - si2 + center_y
    x2 = si1 - co2 + center_x
    y3 = -co1 + si2 + center_y
    x3 = si1 + co2 + center_x
    y4 = co1 + si2 + center_y
    x4 = -si1 + co2 + center_x

    # We have to swap axes for Python from HDevelop
    polygon = [y1, x1, y2, x2, y3, x3, y4, x4]

    return polygon


def convert_polygon2box(polygon):
    """Convert a Python polygon parametrisation of a box to our 5-parameter
    representation.

    This is the inverse of convert_box2polygon.

    Args:
        polygon(list(float)): Polygon coordinates of 4 corner points:
                              [x1, y1, x2, y2, ..., x4, y4]

    Return:
        list(float): Parameters of the bounding box:
                     [center_x, center_y, width, height, angle(deg)]

    Author: Patrick Follmann, MVTec Software GmbH, 2017
    """
    # Swap axes between Python and HDevelop (our representation)
    col1 = polygon[0]
    row1 = polygon[1]
    col2 = polygon[2]
    row2 = polygon[3]
    col3 = polygon[4]
    row3 = polygon[5]
    col4 = polygon[6]
    row4 = polygon[7]

    # Calculate center (row, col)
    row = (row1 + row2 + row3 + row4) / 4.
    col = (col1 + col2 + col3 + col4) / 4.

    # and width and height
    length1 = ((row1 - row2) ** 2 + (col1 - col2) ** 2) ** (0.5) + 1
    length2 = ((row2 - row3) ** 2 + (col2 - col3) ** 2) ** (0.5) + 1

    # Calculate angle
    hor = col2 - col1
    vert = row1 - row2

    # swap length1 and length2 when necessary
    if length1 < length2:
        tmp = length1
        length1 = length2
        length2 = tmp

        hor = col3 - col2
        vert = row2 - row3

    phi = atan2(vert, hor)
    if phi > math.pi / 2.:
        phi -= math.pi
    elif phi <= -math.pi/2.:
        phi += math.pi

    box_params = [row, col, length1, length2, math.degrees(phi)]

    # Consolidate the angle
    consolidate_box_params(box_params)

    return box_params


def consolidate_box_params(box_params):
    """Unify the angle range of box coordinates to be between (-45:45] degrees.

    Args:
        box_params(list(float)): Parameters of the box:
                                [center_x, center_y, width, height, angle(deg)]

    Return:
        list(float): Parameters of the box with angle in (-45:45] degrees:
                     [center_x, center_y, width, height, angle(deg)]

    Author: Patrick Follmann, MVTec Software GmbH, 2017
    """
    phi = box_params[4]
    # Get rid of 360 degree permutations
    phi = phi % 360.0

    # Ensure rotation is in (-180:180]
    if phi > 180.0:
        phi -= 360.0
    elif phi <= -180.0:
        phi += 360.0

    # We have the following cases:
    # phi in (-180:-135] => phi_new = phi + 180.0 NO FLIP
    # phi in (-135: -45] => phi_new = phi + 90.0  FLIP
    # phi in ( -45:  45] =>                       DO NOTHING
    # phi in (  45: 135] => phi_new = phi - 90.0  FLIP
    # phi in ( 135: 180] => phi_new = phi - 180.0 NO FLIP
    flip = ((phi > -135) and (phi <= -45)) or ((phi > 45) and (phi <= 135))
    if phi <= -135:
        phi += 180.0
    elif phi <= -45:
        phi += 90.0
    elif (phi > 45) and (phi <= 135):
        phi -= 90.0
    elif phi > 135:
        phi -= 180.0

    box_params[4] = phi
    # Swap the axes if necessary
    if flip:
        tmp = box_params[2]
        box_params[2] = box_params[3]
        box_params[3] = tmp

    return box_params


def read_segmentation(annotations_dir,
                      annot_name):
    """Read the segmentation from file and return as mask.

    Args:
        annotations_dir(str): directory of the annotations (segmentations)
        annot_name(str): filename of the corresponding annotation.
    Return:
        numpy.array: mask with image-size, where ones indicate the
                     segmentation.

    Author: Patrick Follmann, MVTec Software GmbH 2017.
    """
    # Get the segmentation from file
    segmentation_file = os.path.join(annotations_dir, annot_name)
    # Check if segmentation is available
    if not os.path.isfile(segmentation_file):
        print fmt_str.format('draw_segmentation',
                             'File does not exist: {}'
                             .format(segmentation_file))
        return
    # Read in the segmentation and create a mask with ones where the object is
    mask = np.array(Image.open(segmentation_file).convert("L"))/255

    return mask


def polygon2mask(polygon,
                 mask_size):
    """Convert a polygon to a mask.

    Args:
        polygon(list(float)): polygon as list of corner points as e.g. obtained
                              from convert_box2polygon.
        mask_size(tuple(int)): size of the output mask (width, height).
    Return:
        numpy.array: mask with mask_size, where ones indicate the
                     segmentation.

    Author: Patrick Follmann, MVTec Software GmbH 2017.
    """
    # Create a new empty mask
    img = Image.new('L', mask_size, 0)
    # Round polygon corners to integers
    polygon = [int(round(x)) for x in polygon]
    # Draw the polygon into the mask
    ImageDraw.Draw(img).polygon(polygon, outline=1, fill=1)
    mask = np.array(img)

    return mask


def get_iou(maskA,
            maskB):
    """Calculate the intersection over union for two masks.

    Args:
        maskA(numpy.array): first mask indicating object A.
        maskB(numpy.array): second mask indicating object B.
    Return:
        float: intersection over union for two masks A and B.

    Author: Patrick Follmann, MVTec Software GmbH 2017.
    """
    union = ((maskA > 0) + (maskB > 0)).astype(float)
    intersection = ((maskA > 0) & (maskB > 0)).astype(float)

    # Calculate the intersection over union as quotient of the areas
    iou = np.sum(intersection) / np.sum(union)

    return iou
